@@ -3,8 +3,9 @@ package Moo;
 use strictures 1;
 use Moo::_Utils;
 use B 'perlstring';
+use Sub::Defer ();
 
-our $VERSION = '0.091001'; # 0.91.1
+our $VERSION = '0.091002'; # 0.91.2
 $VERSION = eval $VERSION;
 
 require Moo::sification;
@@ -33,12 +34,10 @@ sub import {
   $MAKERS{$target} = {};
   _install_coderef "${target}::has" => sub {
     my ($name, %spec) = @_;
-    ($MAKERS{$target}{accessor} ||= do {
-      require Method::Generate::Accessor;
-      Method::Generate::Accessor->new
-    })->generate_method($target, $name, \%spec);
     $class->_constructor_maker_for($target)
           ->register_attribute_specs($name, \%spec);
+    $class->_accessor_maker_for($target)
+          ->generate_method($target, $name, \%spec);
   };
   foreach my $type (qw(before after around)) {
     _install_coderef "${target}::${type}" => sub {
@@ -57,6 +56,31 @@ sub import {
   }
 }
 
+sub _accessor_maker_for {
+  my ($class, $target) = @_;
+  return unless $MAKERS{$target};
+  $MAKERS{$target}{accessor} ||= do {
+    my $maker_class = do {
+      if (my $m = do {
+            if (my $defer_target = 
+                  (Sub::Defer::defer_info($target->can('new'))||[])->[0]
+              ) {
+              my ($pkg) = ($defer_target =~ /^(.*)::[^:]+$/);
+              $MAKERS{$pkg} && $MAKERS{$pkg}{accessor};
+            } else {
+              undef;
+            }
+          }) {
+        ref($m);
+      } else {
+        require Method::Generate::Accessor;
+        'Method::Generate::Accessor'
+      }
+    };
+    $maker_class->new;
+  }
+}
+
 sub _constructor_maker_for {
   my ($class, $target, $select_super) = @_;
   return unless $MAKERS{$target};
@@ -84,13 +108,10 @@ sub _constructor_maker_for {
         $moo_constructor = 1; # no other constructor, make a Moo one
       }
     };
-    Method::Generate::Constructor
+    ($con ? ref($con) : 'Method::Generate::Constructor')
       ->new(
         package => $target,
-        accessor_generator => do {
-          require Method::Generate::Accessor;
-          Method::Generate::Accessor->new;
-        },
+        accessor_generator => $class->_accessor_maker_for($target),
         construction_string => (
           $moo_constructor
             ? ($con ? $con->construction_string : undef)
